@@ -20,7 +20,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.31';
+$VERSION = '1.32';
 
 # older Casio maker notes (ref 1)
 %Image::ExifTool::Casio::Main = (
@@ -486,8 +486,34 @@ $VERSION = '1.31';
             '3 0 0' => 'Vivid Landscape',
             # have also seen '1 1 1', '2 2 4', '4 3 3', '4 4 4'
             # '0 0 14' and '0 0 42' - premium auto night shot (EX-Z2300)
+            # and '0 0 2' for Art HDR
         },
     },
+    0x2089 => [ #PH
+        {
+            Name => 'FaceInfo1',
+            Condition => '$$valPt =~ /^(\0\0|.\x02\x80\x01\xe0)/s', # (EX-H5)
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Casio::FaceInfo1',
+                ByteOrder => 'BigEndian',
+            },
+        },{
+            Name => 'FaceInfo2',
+            Condition => '$$valPt =~ /^\x02\x01/', # (EX-H20G,EX-ZR100)
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Casio::FaceInfo2',
+                ByteOrder => 'LittleEndian',
+            },
+        },{
+            Name => 'FaceInfoUnknown',
+            Unknown => 1,
+        },
+    ],
+    # 0x208a - also some sort of face detection information - PH
+    0x211c => { #PH
+        Name => 'FacesDetected',
+        Format => 'int8u',
+    },
     0x3000 => {
         Name => 'RecordMode',
         Writable => 'int16u',
@@ -570,7 +596,7 @@ $VERSION = '1.31';
         PrintConv => {
             0 => 'Off',
             1 => 'On',
-            # have seen 2, 3(portrait) and 5(auto mode)
+            # have seen 2(EX-Z35 macro/portrait), 3(portrait) and 5(auto mode)
             4 => 'Face Recognition', # "Family First"
         },
     },
@@ -634,9 +660,12 @@ $VERSION = '1.31';
         PrintConv => {
             0 => 'Normal',
             8 => 'Silent Movie',
+            39 => 'HDR', # (EX-ZR10)
             45 => 'Premium Auto', # (EX-2300)
             47 => 'Painting', # (EX-2300)
             49 => 'Crayon Drawing', # (EX-2300)
+            51 => 'Panorama', # (EX-ZR10)
+            52 => 'Art HDR', # (EX-ZR10,EX-Z3000)
         },
     },
     0x301c => { #3
@@ -656,7 +685,7 @@ $VERSION = '1.31';
             0 => 'Off',
             1 => 'On',
             2 => 'Best Shot',
-            # 3 observed in MOV videos (EX-V7)
+            # 3 observed in MOV videos (EX-V7, EX-TR100)
             # (newer models write 2 numbers here - PH)
             '0 0' => 'Off', #PH
             # have seen '0 1' for EX-Z2000 which has 5 modes: Auto, Camera AS, Image AS, DEMO, Off
@@ -665,6 +694,7 @@ $VERSION = '1.31';
             '20 0' => 'High Sensitivity', #PH (EX-Z77)
             '0 3' => 'CCD Shift', #PH (guess)
             '2 3' => 'High Speed Anti-Shake', #PH (EX-FC150)
+            # have also seen: '2 0', '16 1'            
         },
     },
     0x302a => { #PH (EX-Z450)
@@ -742,18 +772,186 @@ $VERSION = '1.31';
         ValueConvInv => '$val <= 60 ? $val * 1000 : int($val) . " 0"',
     },
     # 0x4002 - AVI videos, related to video quality or size - PH
-    0x4003 => { #PH (AVI videos)
+    0x4003 => { #PH (AVI and MOV videos)
         Name => 'VideoQuality',
         Writable => 'int16u',
         PrintConv => {
             1 => 'Standard',
             # 2 - could this be LP?
-            3 => 'HD',
+            3 => 'HD (720p)',
+            4 => 'Full HD (1080p)', # (EX-ZR10, 30fps 1920x1080)
             5 => 'Low', # used in High Speed modes
         },
     },
 );
 
+# face detection information (ref PH) (EX-H5)
+%Image::ExifTool::Casio::FaceInfo1 = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+    WRITABLE => 1,
+    FIRST_ENTRY => 0,
+    DATAMEMBER => [ 0 ],
+    NOTES => 'Face-detect tags extracted from models such as the EX-H5.',
+    0x00 => { # (NC)
+        Name => 'FacesDetected',
+        DataMember => 'FacesDetected',
+        RawConv => '$$self{FacesDetected} = $val',
+    },
+    0x01 => {
+        Name => 'FaceDetectFrameSize',
+        Condition => '$$self{FacesDetected} >= 1', # (otherwise zeros)
+        Format => 'int16u[2]',
+    },
+    0x0d => {
+        Name => 'Face1Position',
+        Condition => '$$self{FacesDetected} >= 1',
+        Format => 'int16u[4]',
+        Notes => q{
+            left, top, right and bottom of detected face in coordinates of
+            FaceDetectFrameSize, with increasing Y downwards
+        },
+    },
+    # decoding NOT CONFIRMED (NC) for faces 2-10!
+    0x7c => {
+        Name => 'Face2Position',
+        Condition => '$$self{FacesDetected} >= 2',
+        Format => 'int16u[4]',
+    },
+    0xeb => {
+        Name => 'Face3Position',
+        Condition => '$$self{FacesDetected} >= 3',
+        Format => 'int16u[4]',
+    },
+    0x15a => {
+        Name => 'Face4Position',
+        Condition => '$$self{FacesDetected} >= 4',
+        Format => 'int16u[4]',
+    },
+    0x1c9 => {
+        Name => 'Face5Position',
+        Condition => '$$self{FacesDetected} >= 5',
+        Format => 'int16u[4]',
+    },
+    0x238 => {
+        Name => 'Face6Position',
+        Condition => '$$self{FacesDetected} >= 6',
+        Format => 'int16u[4]',
+    },
+    0x2a7 => {
+        Name => 'Face7Position',
+        Condition => '$$self{FacesDetected} >= 7',
+        Format => 'int16u[4]',
+    },
+    0x316 => {
+        Name => 'Face8Position',
+        Condition => '$$self{FacesDetected} >= 8',
+        Format => 'int16u[4]',
+    },
+    0x385 => {
+        Name => 'Face9Position',
+        Condition => '$$self{FacesDetected} >= 9',
+        Format => 'int16u[4]',
+    },
+    0x3f4 => {
+        Name => 'Face10Position',
+        Condition => '$$self{FacesDetected} >= 10',
+        Format => 'int16u[4]',
+    },
+);
+
+# face detection information (ref PH) (EX-ZR100)
+%Image::ExifTool::Casio::FaceInfo2 = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+    WRITABLE => 1,
+    FIRST_ENTRY => 0,
+    DATAMEMBER => [ 2 ],
+    NOTES => 'Face-detect tags extracted from models such as the EX-H20G and EX-ZR100.',
+    0x02 => {
+        Name => 'FacesDetected',
+        DataMember => 'FacesDetected',
+        RawConv => '$$self{FacesDetected} = $val',
+    },
+    0x04 => {
+        Name => 'FaceDetectFrameSize',
+        Condition => '$$self{FacesDetected} >= 1',
+        Format => 'int16u[2]',
+    },
+    0x08 => {
+        Name => 'FaceOrientation',
+        Condition => '$$self{FacesDetected} >= 1',
+        PrintConv => {
+            0 => 'Horizontal (normal)',
+            1 => 'Rotate 90 CW',
+            2 => 'Rotate 270 CW',
+            3 => 'Rotate 180', # (NC)
+        },
+        Notes => 'orientation of face relative to unrotated image',
+    },
+    # 0x0a - FaceDetectFrameSize again
+    # 0x11 - Face1Detected flag (1=detected)
+    0x18 => {
+        Name => 'Face1Position',
+        Condition => '$$self{FacesDetected} >= 1',
+        Format => 'int16u[4]',
+        Notes => q{
+            left, top, right and bottom of detected face in coordinates of
+            FaceDetectFrameSize, with increasing Y downwards
+        },
+    },
+    # 0x45 - Face2Detected, etc...
+    0x4c => {
+        Name => 'Face2Position',
+        Condition => '$$self{FacesDetected} >= 2',
+        Format => 'int16u[4]',
+    },
+    0x80 => {
+        Name => 'Face3Position',
+        Condition => '$$self{FacesDetected} >= 3',
+        Format => 'int16u[4]',
+    },
+    0xb4 => {
+        Name => 'Face4Position',
+        Condition => '$$self{FacesDetected} >= 4',
+        Format => 'int16u[4]',
+    },
+    0xe8 => {
+        Name => 'Face5Position',
+        Condition => '$$self{FacesDetected} >= 5',
+        Format => 'int16u[4]',
+    },
+    0x11c => {
+        Name => 'Face6Position',
+        Condition => '$$self{FacesDetected} >= 6',
+        Format => 'int16u[4]',
+    },
+    0x150 => {
+        Name => 'Face7Position',
+        Condition => '$$self{FacesDetected} >= 7',
+        Format => 'int16u[4]',
+    },
+    0x184 => {
+        Name => 'Face8Position',
+        Condition => '$$self{FacesDetected} >= 8',
+        Format => 'int16u[4]',
+    },
+    0x1b8 => {
+        Name => 'Face9Position',
+        Condition => '$$self{FacesDetected} >= 9',
+        Format => 'int16u[4]',
+    },
+    0x1ec => {
+        Name => 'Face10Position',
+        Condition => '$$self{FacesDetected} >= 10',
+        Format => 'int16u[4]',
+    },
+);
+
 # Casio APP1 QVCI segment found in QV-7000SX images (ref PH)
 %Image::ExifTool::Casio::QVCI = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
@@ -830,7 +1028,7 @@ Casio maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
