@@ -26,7 +26,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.24';
+$VERSION = '1.25';
 
 sub ProcessPNG_tEXt($$$);
 sub ProcessPNG_iTXt($$$);
@@ -541,7 +541,6 @@ sub FoundPNG($$$$;$$$$)
     my ($exifTool, $tagTablePtr, $tag, $val, $compressed, $outBuff, $enc, $lang) = @_;
     return 0 unless defined $val;
     my $verbose = $exifTool->Options('Verbose');
-    my $out = $exifTool->Options('TextOut');
     my $id = $tag;  # generate tag ID which include language code
     if ($lang) {
         # case of language code must be normalized since they are case insensitive
@@ -662,9 +661,9 @@ sub FoundPNG($$$$;$$$$)
                     # (also handle case of tEXt tags written with lowercase first letter)
                     delete $exifTool->{ADD_PNG}->{ucfirst($id)};
                     my $nvHash = $exifTool->GetNewValueHash($tagInfo);
-                    $isOverwriting = Image::ExifTool::IsOverwriting($nvHash);
+                    $isOverwriting = $exifTool->IsOverwriting($nvHash);
                     if (defined $deflateErr) {
-                        $newVal = Image::ExifTool::GetNewValues($nvHash);
+                        $newVal = $exifTool->GetNewValues($nvHash);
                         # can only write tag now if always overwriting
                         if ($isOverwriting > 0) {
                             $val = '<deflate error>';
@@ -674,10 +673,10 @@ sub FoundPNG($$$$;$$$$)
                         }
                     } else {
                         if ($isOverwriting < 0) {
-                            $isOverwriting = Image::ExifTool::IsOverwriting($nvHash, $val);
+                            $isOverwriting = $exifTool->IsOverwriting($nvHash, $val);
                         }
                         # (must get new value after IsOverwriting() in case it was shifted)
-                        $newVal = Image::ExifTool::GetNewValues($nvHash);
+                        $newVal = $exifTool->GetNewValues($nvHash);
                     }
                 }
                 if ($isOverwriting) {
@@ -1072,7 +1071,7 @@ and JNG (JPEG Network Graphics) images.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
