@@ -12,17 +12,17 @@
 package Image::ExifTool::SigmaRaw;
 
 use strict;
-use vars qw($VERSION);
+use vars qw($VERSION %sigmaLensTypes);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.14';
+$VERSION = '1.16';
 
 sub ProcessX3FHeader($$$);
 sub ProcessX3FDirectory($$$);
 sub ProcessX3FProperties($$$);
 
 # sigma LensType lookup (ref PH)
-my %sigmaLensTypes = (
+%sigmaLensTypes = (
     # 0 => 'Sigma 50mm F2.8 EX Macro', (0 used for other lenses too)
     # 8 - 18-125mm LENSARANGE@18mm=22-4
     16 => 'Sigma 18-50mm F3.5-5.6 DC',
@@ -38,8 +38,9 @@ my %sigmaLensTypes = (
     # 70-200mm F2.8 EX DG Macro (HSM) - Minimum focus distance reduced to 1m (2006)
     # 70-200mm F2.8 EX DG Macro HSM II - Improved optical performance (2007)
     169 => 'Sigma 18-50mm F2.8 EX DC', #(NC)
-    '100' => 'Sigma 24-70mm f/2.8 DG Macro', # (SD15)
-    8900 => 'Sigma 70-300mm f/4-5.6 DG OS', # (SD15)
+    581 => 'Sigma 18-50mm F2.8 EX DC Macro', # (SD1)
+    8900 => 'Sigma 70-300mm F4-5.6 DG OS', # (SD15)
+    'A100' => 'Sigma 24-70mm F2.8 DG Macro', # (SD15)
 );
 
 # main X3F sections (plus header stuff)
