@@ -8,8 +8,15 @@
 package Image::ExifTool::CanonRaw;
 
 use strict;
+use vars qw($VERSION $AUTOLOAD %crwTagFormat);
 use Image::ExifTool::Fixup;
 
+# map for adding directories to CRW
+my %crwMap = (
+    XMP      => 'CanonVRD',
+    CanonVRD => 'Trailer',
+);
+
 # mappings to from RAW tagID to MakerNotes tagID
 # (Note: upper two bits of RawTagID are zero)
 my %mapRawTag = (
@@ -22,7 +29,7 @@ my %mapRawTag = (
     0x102a => 0x04, # CanonShotInfo
     0x102d => 0x01, # CanonCameraSettings
     0x1033 => 0x0f, # CanonCustomFunctions (only verified for 10D)
-    0x1038 => 0x12, # CanonPictureInfo
+    0x1038 => 0x12, # CanonAFInfo
     0x1039 => 0x13,
     0x1093 => 0x93,
     0x10a8 => 0xa8,
@@ -75,7 +82,6 @@ sub BuildMakerNotes($$$$$$)
     # special case: ignore user comment because it gets saved in EXIF
     # (and has the same raw tagID as CanonFileDescription)
     return if $tagInfo and $$tagInfo{Name} eq 'UserComment';
-    my $tagType = ($rawTag >> 8) & 0x38;
     my $format = $Image::ExifTool::Exif::formatNumber{$formName};
     my $fsiz = $Image::ExifTool::Exif::formatSize[$format];
     my $size = length($$valuePt);
@@ -164,7 +170,6 @@ sub SaveMakerNotes($)
 sub CheckCanonRaw($$$)
 {
     my ($exifTool, $tagInfo, $valPtr) = @_;
-    Image::ExifTool::GenerateTagIDs($$tagInfo{Table});
     my $tagName = $$tagInfo{Name};
     if ($tagName eq 'JpgFromRaw' or $tagName eq 'ThumbnailImage') {
         unless ($$valPtr =~ /^\xff\xd8/ or $exifTool->Options('IgnoreMinorErrors')) {
@@ -191,16 +196,31 @@ sub WriteCR2($$$)
 {
     my ($exifTool, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt} or return 0;
-    my $raf = $$dirInfo{RAF} or return 0;
     my $outfile = $$dirInfo{OutFile} or return 0;
+    $$dirInfo{RAF} or return 0;
 
     # check CR2 signature
-    return 0 if $$dataPt !~ /^.{8}CR\x02\0/s and
-        $exifTool->Error("Unsupported Canon RAW file. May cause problems if rewritten", 1);
+    if ($$dataPt !~ /^.{8}CR\x02\0/s) {
+        my ($msg, $minor);
+        if ($$dataPt =~ /^.{8}CR/s) {
+            $msg = 'Unsupported Canon RAW file. May cause problems if rewritten';
+            $minor = 1;
+        } elsif ($$dataPt =~ /^.{8}\xba\xb0\xac\xbb/s) {
+            $msg = 'Can not currently write Canon 1D RAW images';
+        } else {
+            $msg = 'Unrecognized Canon RAW file';
+        }
+        return 0 if $exifTool->Error($msg, $minor);
+    }
+
     # CR2 has a 16-byte header
     $$dirInfo{NewDataPos} = 16;
     my $newData = $exifTool->WriteDirectory($dirInfo, $tagTablePtr);
-    return 0 unless defined $newData and $$dirInfo{LastIFD};
+    return 0 unless defined $newData;
+    unless ($$dirInfo{LastIFD}) {
+        $exifTool->Error("CR2 image IFD may not be deleted");
+        return 0;
+    }
 
     if (length($newData)) {
         # build 16 byte header for Canon RAW file
@@ -217,56 +237,44 @@ sub WriteCR2($$$)
             $exifTool->CopyImageData($$dirInfo{ImageData}, $outfile) or return 0;
             delete $$dirInfo{ImageData};
         }
-        # write anything after the end of a Canon RAW image
-        if ($exifTool->{TIFF_END}) {
-            $raf->Seek($exifTool->{TIFF_END}, 0) or return 0;
-            my $extra = 0;
-            while ($raf->Read($newData, 65536)) {
-                Write($outfile, $newData) or return 0;
-                $extra += length $newData;
-            }
-            if ($extra) {
-                $exifTool->VPrint(0, "Note: $extra extra bytes copied after normal end of CR2\n");
-            }
-        }
     }
     return 1;
 }
 
 #------------------------------------------------------------------------------
-# Write CanonRaw information
+# Write CanonRaw (CRW) information
 # Inputs: 0) ExifTool object reference, 1) source dirInfo reference,
 #         2) tag table reference
-# Returns: undefined on error, else success message
+# Returns: true on success
 # Notes: Increments ExifTool CHANGED flag for each tag changed This routine is
 # different from all of the other write routines because Canon RAW files are
 # designed well!  So it isn't necessary to buffer the data in memory before
 # writing it out.  Therefore this routine doesn't return the directory data as
 # the rest of the Write routines do.  Instead, it writes to the dirInfo
-# Outfile on the fly --> much faster, efficient, and less demanding on memory!
+# OutFile on the fly --> much faster, efficient, and less demanding on memory!
 sub WriteCanonRaw($$$)
 {
     my ($exifTool, $dirInfo, $tagTablePtr) = @_;
     $exifTool or return 1;    # allow dummy access to autoload this package
     my $blockStart = $$dirInfo{DirStart};
     my $blockSize = $$dirInfo{DirLen};
-    my $raf = $$dirInfo{RAF} or return undef;
-    my $outfile = $$dirInfo{OutFile} or return undef;
-    my $outPos = $$dirInfo{OutPos} or return undef;
+    my $raf = $$dirInfo{RAF} or return 0;
+    my $outfile = $$dirInfo{OutFile} or return 0;
+    my $outPos = $$dirInfo{OutPos} or return 0;
     my $outBase = $outPos;
     my $verbose = $exifTool->Options('Verbose');
     my $out = $exifTool->Options('TextOut');
     my ($buff, $tagInfo);
 
     # 4 bytes at end of block give directory position within block
-    $raf->Seek($blockStart+$blockSize-4, 0) or return undef;
-    $raf->Read($buff, 4) == 4 or return undef;
+    $raf->Seek($blockStart+$blockSize-4, 0) or return 0;
+    $raf->Read($buff, 4) == 4 or return 0;
     my $dirOffset = Get32u(\$buff,0) + $blockStart;
-    $raf->Seek($dirOffset, 0) or return undef;
-    $raf->Read($buff, 2) == 2 or return undef;
+    $raf->Seek($dirOffset, 0) or return 0;
+    $raf->Read($buff, 2) == 2 or return 0;
     my $entries = Get16u(\$buff,0);             # get number of entries in directory
     # read the directory (10 bytes per entry)
-    $raf->Read($buff, 10 * $entries) == 10 * $entries or return undef;
+    $raf->Read($buff, 10 * $entries) == 10 * $entries or return 0;
     my $newDir = '';
 
     # get hash of new information keyed by tagID
@@ -300,11 +308,16 @@ sub WriteCanonRaw($$$)
             $tagInfo = $$newTags{$addTag};
             my $newVal = $exifTool->GetNewValues($tagInfo);
             if (defined $newVal) {
+                # pad value to an even length (Canon ImageBrowser and ZoomBrowser
+                # version 6.1.1 have problems with odd-sized embedded JPEG images
+                # even if the value is padded to maintain alignment, so do this
+                # before calculating the size for the directory entry)
+                $newVal .= "\0" if length($newVal) & 0x01;
                 # add new directory entry
                 $newDir .= Set16u($addTag) . Set32u(length($newVal)) .
                            Set32u($outPos - $outBase);
                 # write new value data
-                Write($outfile, $newVal) or return undef;
+                Write($outfile, $newVal) or return 0;
                 $outPos += length($newVal);     # update current position
                 $verbose > 1 and print $out "    + CanonRaw:$$tagInfo{Name}\n";
                 ++$exifTool->{CHANGED};
@@ -313,7 +326,7 @@ sub WriteCanonRaw($$$)
             $delTag{$addTag} = 1;
         }
         last unless defined $tag;           # all done if no more directory entries
-        return undef if $tag & 0x8000;      # top bit should not be set
+        return 0 if $tag & 0x8000;          # top bit should not be set
         my $tagID = $tag & 0x3fff;          # get tag ID
         my $tagType = ($tag >> 8) & 0x38;   # get tag type
         my $valueInDir = ($tag & 0x4000);   # flag for value in directory
@@ -350,17 +363,17 @@ sub WriteCanonRaw($$$)
                     OutPos   => $outPos,
                 );
                 my $result = $exifTool->WriteDirectory(\%subdirInfo, $tagTablePtr);
-                return undef unless $result;
+                return 0 unless $result;
                 # set size and pointer for this new directory
                 $size = $subdirInfo{OutPos} - $outPos;
                 $valuePtr = $outPos - $outBase;
                 $outPos = $subdirInfo{OutPos};
             } else {
                 # verify that the value data is within this block
-                $valuePtr + $size <= $blockSize or return undef;
+                $valuePtr + $size <= $blockSize or return 0;
                 # read value from file
-                $raf->Seek($ptr, 0) or return undef;
-                $raf->Read($value, $size) == $size or return undef;
+                $raf->Seek($ptr, 0) or return 0;
+                $raf->Read($value, $size) == $size or return 0;
             }
         }
         # set count from tagInfo count if necessary
@@ -375,7 +388,7 @@ sub WriteCanonRaw($$$)
             if ($subdir and $$subdir{TagTable}) {
                 my $name = $$tagInfo{Name};
                 my $newTagTable = Image::ExifTool::GetTagTable($$subdir{TagTable});
-                return undef unless $newTagTable;
+                return 0 unless $newTagTable;
                 my $subdirStart = 0;
                 #### eval Start ()
                 $subdirStart = eval $$subdir{Start} if $$subdir{Start};
@@ -416,9 +429,9 @@ sub WriteCanonRaw($$$)
                 } else {
                     $oldVal = $value;
                 }
-                my $newValueHash = $exifTool->GetNewValueHash($tagInfo);
-                if (Image::ExifTool::IsOverwriting($newValueHash, $oldVal)) {
-                    my $newVal = Image::ExifTool::GetNewValues($newValueHash);
+                my $nvHash = $exifTool->GetNewValueHash($tagInfo);
+                if ($exifTool->IsOverwriting($nvHash, $oldVal)) {
+                    my $newVal = $exifTool->GetNewValues($nvHash);
                     my $verboseVal;
                     $verboseVal = $newVal if $verbose > 1;
                     # convert to specified format if necessary
@@ -428,12 +441,8 @@ sub WriteCanonRaw($$$)
                     if (defined $newVal) {
                         $value = $newVal;
                         ++$exifTool->{CHANGED};
-                        if ($verbose > 1) {
-                            my $oldStr = $exifTool->Printable($oldVal);
-                            my $newStr = $exifTool->Printable($verboseVal);
-                            print $out "    - CanonRaw:$$tagInfo{Name} = '$oldStr'\n";
-                            print $out "    + CanonRaw:$$tagInfo{Name} = '$newStr'\n";
-                        }
+                        $exifTool->VerboseValue("- CanonRaw:$$tagInfo{Name}", $oldVal);
+                        $exifTool->VerboseValue("+ CanonRaw:$$tagInfo{Name}", $verboseVal);
                     }
                 }
             }
@@ -468,7 +477,7 @@ sub WriteCanonRaw($$$)
             $value .= "\0" if $size & 0x01;
             $valuePtr = $outPos - $outBase;
             # write out value data
-            Write($outfile, $value) or return undef;
+            Write($outfile, $value) or return 0;
             $outPos += length($value);  # update current position in outfile
         }
         # create new directory entry
@@ -478,12 +487,14 @@ sub WriteCanonRaw($$$)
     $entries = length($newDir) / 10;
     $newDir = Set16u($entries) . $newDir . Set32u($outPos - $outBase);
     # write directory data
-    Write($outfile, $newDir) or return undef;
+    Write($outfile, $newDir) or return 0;
 
     # update current output file position in dirInfo
     $$dirInfo{OutPos} = $outPos + length($newDir);
+    # save outfile directory start (needed for rewriting VRD trailer)
+    $$dirInfo{OutDirStart} = $outPos - $outBase;
 
-    return 'Success';
+    return 1;
 }
 
 #------------------------------------------------------------------------------
@@ -499,26 +510,36 @@ sub WriteCRW($$)
     my $rtnVal = 0;
     my ($buff, $err, $sig);
 
-    $raf->Read($buff,2) == 2 or return 0;
-    SetByteOrder($buff)      or return 0;
-    $raf->Read($buff,4) == 4 or return 0;
-    $raf->Read($sig,8) == 8  or return 0;   # get file signature
-    $sig eq 'HEAPCCDR'       or return 0;   # validate signature
+    $raf->Read($buff,2) == 2    or return 0;
+    SetByteOrder($buff)         or return 0;
+    $raf->Read($buff,4) == 4    or return 0;
+    $raf->Read($sig,8) == 8     or return 0;   # get file signature
+    $sig =~ /^HEAP(CCDR|JPGM)/  or return 0;   # validate signature
+    my $type = $1;
     my $hlen = Get32u(\$buff, 0);   # get header length
 
+    if ($exifTool->{DEL_GROUP}->{MakerNotes}) {
+        if ($type eq 'CCDR') {
+            $exifTool->Error("Can't delete MakerNotes group in CRW file");
+            return 0;
+        } else {
+            ++$exifTool->{CHANGED};
+            return 1;
+        }
+    }
+    # make XMP the preferred group for CRW files
+    if ($$exifTool{FILE_TYPE} eq 'CRW') {
+        $exifTool->InitWriteDirs(\%crwMap, 'XMP');
+    }
+
     # write header
-    $raf->Seek(0, 0)         or return 0;
+    $raf->Seek(0, 0)            or return 0;
     $raf->Read($buff, $hlen) == $hlen or return 0;
     Write($outfile, $buff) or $err = 1;
 
-    $raf->Seek(0, 2)         or return 0;   # seek to end of file
+    $raf->Seek(0, 2)            or return 0;   # seek to end of file
     my $filesize = $raf->Tell() or return 0;
 
-    if ($exifTool->{DEL_GROUP}) {
-        $exifTool->Error("Can't delete groups in CRW file");
-        return 0;
-    }
-
     # build directory information for main raw directory
     my %dirInfo = (
         DataLen  => 0,
@@ -530,23 +551,51 @@ sub WriteCRW($$)
         OutFile  => $outfile,
         OutPos   => $hlen,
     );
-
     # process the raw directory
     my $tagTablePtr = Image::ExifTool::GetTagTable('Image::ExifTool::CanonRaw::Main');
-    my $msg = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr);
-    if ($msg) {
-        if ($err) {
-            $rtnVal = -1;
-        } else {
-            $rtnVal = 1;    # success!
+    my $success = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr);
+
+    my $trailPt;
+    while ($success) {
+        # check to see if trailer(s) exist(s)
+        my $trailInfo = Image::ExifTool::IdentifyTrailer($raf) or last;
+        # rewrite the trailer(s)
+        $buff = '';
+        $$trailInfo{OutFile} = \$buff;
+        $success = $exifTool->ProcessTrailers($trailInfo) or last;
+        $trailPt = $$trailInfo{OutFile};
+        # nothing to write if trailers were deleted
+        undef $trailPt if length($$trailPt) < 4;
+        last;
+    }
+    if ($success) {
+        # add CanonVRD trailer if writing as a block
+        $trailPt = $exifTool->AddNewTrailers($trailPt,'CanonVRD');
+        if (not $trailPt and $$exifTool{ADD_DIRS}{CanonVRD}) {
+            # create CanonVRD from scratch if necessary
+            my $outbuff = '';
+            my $saveOrder = GetByteOrder();
+            require Image::ExifTool::CanonVRD;
+            if (Image::ExifTool::CanonVRD::ProcessCanonVRD($exifTool, { OutFile => \$outbuff }) > 0) {
+                $trailPt = \$outbuff;
+            }
+            SetByteOrder($saveOrder);
         }
+        # write trailer
+        if ($trailPt) {
+            # must append DirStart pointer to end of trailer
+            my $newDirStart = Set32u($dirInfo{OutDirStart});
+            my $len = length $$trailPt;
+            my $pad = ($len & 0x01) ? ' ' : ''; # add pad byte if necessary
+            Write($outfile, $pad, substr($$trailPt,0,$len-4), $newDirStart) or $err = 1;
+        }
+        $rtnVal = $err ? -1 : 1;
     } else {
         $exifTool->Error('Error rewriting CRW file');
     }
     return $rtnVal;
 }
 
-
 1; # end
 
 __END__
@@ -573,7 +622,7 @@ files, and would lead to far fewer problems with corrupted metadata.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
